@@ -3,7 +3,7 @@
 /// Natural number
 ///
 ///<a href="https://en.wikipedia.org/wiki/Natural_number">https://en.wikipedia.org/wiki/Natural_number</a>
-pub trait Natural
+pub trait Natural: Eq
 {
 }
 
